﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using RTedge;
using EgMaint;

namespace EgMGather
{
    /// <summary>
    /// RT-Edgeの操作クラス
    /// </summary>
    public partial class EdgeService
    {
        private static EdgeService service = new EdgeService(); // EdgeServiceクラス生成
        MainWindow _Main = null;                                // 親クラス
        Gather gather = null;                                   // 収集クラス
        /// <summary>
        /// RT-edge container common
        /// </summary>
        public edge_API EGAPI;                                              // edge APIのインスタンス
        public string ServiceRealName = "";                                 // xmlで指定したサービスコンテナ名 EgInit後設定
        public string DefServiceName = "EgMGather";                         // サービスコンテナ名デフォルト値 EgInit前で使用
        public ServiceLiveIndicatorTimer ServiceIndLiveTimer;               // サービスインジケータ用のタイマ
        public byte InitState = CDefine.INIT_STS_IDLE;                      // サービス初期化状態(0:未初期化, 1:初期化中, 2:初期化中(Run受信済み), 3:初期化完了)

        public List<string> GatherFileList = new List<string>();            // 収集ファイルリスト

        
        /// <summary>
        /// 入出力スレッド情報
        /// </summary>
        public class IO_TRDINFO
        {
            public SemaphoreSlim hTrigSem;              // 入出力トリガセマフォハンドル
            public CancellationTokenSource threadCTS;   // 入出力スレッドキャンセル処理用変数
            public UInt32 uiOpeCnt;                     // スレッド動作回数
        }
        IO_TRDINFO ThreadInfo_Gather = null;                   //収集用スレッドテーブル
        
        /// <summary>
        /// コンストラクタ(private)
        /// </summary>
        private EdgeService() { }

        /// <summary>
        /// インスタンス取得関数
        /// --------------------------------------------------------
        /// 使用する際は下記のようにインスタンスを取得してください。
        /// EdgeService service = EdgeService.GetInstance();
        /// </summary>
        /// <returns></returns>
        public static EdgeService GetInstance()
        {
            return service;
        }
        /// <summary>
        /// サービス起動時処理
        /// </summary>
        /// <returns></returns>
        public int Startup(MainWindow Main)
        {
            try
            {
                SetInitStatus(CDefine.INIT_STS_INIT);           // 初期化中に更新

                _Main = Main;

                // RT-edgeの準備
                EGAPI = new edge_API();

                // RT-edgeユーザハンドラ登録
                edge_API.EDGECONFIG config = EGAPI.EDGE_CONFIG_DEFAULT;
                config.mInstMessFunc = MsgHandler;

                // RT-edgeフレームワーク初期化
                var ret = EGAPI.EgInit(config);
                if (ret == 0)
                {
                    // サービス名の設定
                    if (EGAPI.EGFW.ServiceRealName != null)
                        ServiceRealName = EGAPI.EGFW.ServiceRealName;

                    gather = new Gather();        // gatherクラス生成

                    CreateMntTag();               //内部タグ構造体の作成

                    // ライブインジケータタグの周期機能 / Service Live counter timer
                    ServiceIndLiveTimer = new ServiceLiveIndicatorTimer(this, 1000);

                    SetErrorID(CDefine.SUCCESS);  //エラーコード設定

                    // インジケータタグの準備
                    CreateIndicater();

                    // XMLプロパティタグ設定の取り出し
                    GetProperty();
                    
                    // TagRefからファイルリストを作成する場合の処理 必要あればここで実装 
                    //GetGaterListFromTagRef();
                    
                    // GatherFileListの読みこみ リストが無い場合は実行したくない為ここで数える
                    ret = GetGatherFileList();
                    if (ret == 0)
                    {
                        ret = CheckOutFolder(); //フォルダのチェックも実施 ある場合のみ実行
                        if (ret == 0)
                        {
                            // 収集スレッド生成
                            ThreadInfo_Gather = CreateOutThreadInfo();                       // 収集スレッド生成
                            if (GatherFileList.Count > 0 && (UInt64)Cycle.Value > 0)                // リストがある かつ サイクル周期の指定がある場合のみスレッド作成
                                ThreadGatherService(ThreadInfo_Gather.threadCTS.Token, ThreadInfo_Gather);  // 収集スレッド生成

                            // ライブインジケータタグの周期更新スタート / Service Live counter timer
                            ServiceIndLiveTimer.RefleshStart();
                            if (CheckInitRun())                                 // 初期化中(RUN受信済み)の場合
                            {
                                // 初期化途中に EM_SERVICE_RUN 受信していた場合は、ここに入る
                                SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                                StartService();                                 // サービス開始処理実行
                            }
                            SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                        }
                        else
                        {
                            SetErrorID((uint)ret);
                        }
                    }
                    else
                    {
                        PrintLog("ERR", "Does not set Gather File List at ECI(XML). Please check the ECI(XML) File.");
                        SetErrorID((uint)ret);
                    }
                }
                else
                {
                    PrintLog("ERR", "Failed to initialize. Please check the environment.");
                    SetErrorID((uint)ret);
                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                SetErrorID((uint)0xFFFFFFFF);
                return -1;
            }
        }


        /// <summary>
        /// 自己のサービス名を返す
        /// </summary>
        /// <returns></returns>
        public String GetServiceName()
        {
            if (ServiceRealName.Length > 0) //XMLで指定したサービス名 EgInit後に設定 設定がない場合はデフォルト値を返す
                return ServiceRealName;
            return DefServiceName; //"EgBM"
        }
        /// <summary>
        /// サービス+タグ取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetServiceTagName(string TagTailName)
        {
            return String.Format("SERVICE.{0}.{1}", GetServiceName(), TagTailName);
        }
        /// <summary>
        /// システムタグ名取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetSystemTagName(string TagTailName)
        {
            return String.Format("EDGESYSTEM.{0}", TagTailName);
        }
        /// <summary>
        /// 内部で保持するタグクラスの作成
        /// </summary>
        public void CreateMntTag()
        {
            //タグの定義
            ServiceIndRun = new MntTagCls(GetServiceTagName("Run"), EGDEFINE.egTagDataType.Boolean);　     // インジケータタグ".Run" : bool / Service Run indicator
            ServiceIndStatus = new MntTagCls(GetServiceTagName("Status"), EGDEFINE.egTagDataType.Byte);　  // インジケータタグ".Status" : byte / Service Status indicator
            ServiceIndError = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);　   // インジケータタグ".Error" : bool / Service Error indicator
            ServiceIndLive = new MntTagCls(GetServiceTagName("Live"), EGDEFINE.egTagDataType.UInt32);　    // インジケータタグ".Live" : UInt32 / Service Live counter indicator

            /// <summary>
            /// プロパティタグ
            /// </summary>
            Cycle = new MntTagCls(GetServiceTagName("Cycle"), EGDEFINE.egTagDataType.UInt64);               // プロパティタグ".Cycle" : UInt64 診断機能用 診断周期(秒)
            AutoRun = new MntTagCls(GetServiceTagName("AutoRun"), EGDEFINE.egTagDataType.Boolean);             // プロパティタグ".AutoRun" : bool 動作自動スタートの有効有無
            Error = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);               // インジケータタグ".Error" : bool エラー状態の有効有無
            ErrorID = new MntTagCls(GetServiceTagName("ErrorID"), EGDEFINE.egTagDataType.UInt32);             // インジケータタグ".ErrorID" UInt32 実行失敗時のエラーコード
            ExecuteCnt = new MntTagCls(GetServiceTagName("ExecuteCnt"), EGDEFINE.egTagDataType.UInt32);          // 統計情報タグ".ExecuteCnt" UInt32 実行数
            ThreadCnt = new MntTagCls(GetServiceTagName("ThreadCnt"), EGDEFINE.egTagDataType.UInt32);          // 統計情報タグ".ExecuteCnt" UInt32 実行数

            Gather_FolerPath = new MntTagCls(GetServiceTagName("FolerPath"), EGDEFINE.egTagDataType.String);// プロパティタグ 出力フォルダパス String
            Gather_ListPath = new MntTagCls(GetServiceTagName("ListPath"), EGDEFINE.egTagDataType.String);  // プロパティタグ リストパス String

            SkipFirstFlag = new MntTagCls(GetServiceTagName("SkipFirstFlag"), EGDEFINE.egTagDataType.Boolean);             // プロパティタグ".SkipFirstFlag" : bool 収集機能 Cycle1回目を飛ばすか否か

            /// <summary>
        }
        /// <summary>
        /// 初期化状態更新
        /// </summary>
        /// <param name="sts"></param>
        public void SetInitStatus(byte sts)
        {
            InitState = sts;
        }
        /// <summary>
        /// 初期化完了済みか？(true:済み,false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitEnd()
        {
            return (InitState == CDefine.INIT_STS_INITEND);
        }
        /// <summary>
        /// 初期化中(RUN受信済み)か？(true:初期化中(RUN受信済み),false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitRun()
        {
            return (InitState == CDefine.INIT_STS_INITRUN);
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public int StartService()
        {
            try
            {
                ServiceIndRun.Value = true;                           // サービス実行状態に更新
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// ErrorIDの設定
        /// </summary>
        /// <param name="_ErrorID"></param>
        public void SetErrorID(uint _ErrorID)
        {
            ErrorID.Value = _ErrorID;
            if (_ErrorID == 0)  //0の時は成功 Errorフラグも変更
                ServiceIndError.Value = false;
            else
                ServiceIndError.Value = true;
        }
        /// <summary>
        /// インジケータタグの生成
        /// </summary>
        /// <returns></returns>
        public int CreateIndicater()
        {
            int ret = 0;

            // インジケータタグを生成します
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Status", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Error", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.ErrorID", GetServiceName()), (ushort)EGDEFINE.egTagDataType.UInt32, 4, "", "service etc last Error ID");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Run", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Boolean, 1, "", "service runnning");

            //各値初期化
            ServiceIndError.Value = false;
            ErrorID.Value = (UInt32)0;
            ExecuteCnt.Value = (UInt32)0;
            
            return ret;
        }
        /// <summary>
        /// サービスプロパティのXML解釈処理
        /// </summary>
        /// <returns></returns>
        public bool GetProperty()
        {
            bool ret = true;
            ret |= GetTagValueTagCls(ref Cycle, (UInt64)0);// Diag.Cycleオプションの利用（デフォルト 0 周期で収集しない）

            ret |= GetTagValueTagCls(ref Gather_FolerPath, (string)"");// FolerPathオプションの利用（デフォルト バッチプログラムデフォルト カレント）
            ret |= GetTagValueTagCls(ref Gather_ListPath, (string)"");// ListPathオプションの利用（デフォルト バッチプログラムデフォルト カレント プログラム名+lst）

            ret |= GetTagValueTagCls(ref AutoRun, (bool)false);//動作自動スタートの有効有無

            ret |= GetTagValueTagCls(ref SkipFirstFlag, (bool)false);//Cycle1回目を飛ばすか否か
            return ret;
        }
        //
        // TagRefからファイルリストを作成する場合の処理 必要あれば実施
        //
        //public void GetGaterListFromTagRef()
        //{
        //    if (EGAPI.EGFW.tagRef_OUT_List.Count > 0)
        //    {

        //    }
        //}
        /// <summary>
        /// ファイルリストの取得
        /// </summary>
        /// <returns></returns>
        public int GetGatherFileList()
        {
            try
            {
                string ListPath = (string)Gather_ListPath.Value;
                //収集ファイルの指定がある場合のみ実施
                if (ListPath.Length > 0)
                {
                    string ListFullPath = ListPath;
                    // フルパスの取得
                    if (CommonFunc.fnIsValidPath(ref ListFullPath) == false)
                    {
                        PrintLog("ERR", "収集ファイルリストパスが正しくありません。収集機能は実行されません。");
                        return (int)CDefine.ERR_GATHER_FATAL_TARGETFILE;//取得できない場合は終了
                    }
                    // リストファイルの存在チェック
                    if (File.Exists(ListFullPath) == false)
                    {
                        PrintLog("ERR", "収集ファイルリストが存在しません。収集機能は実行されません。");
                        return (int)CDefine.ERR_GATHER_EXIST_TARGETFILE;//取得できない場合は終了
                    }

                    // リストファイルからアーカイブ対象フォルダ・ファイルを抽出する
                    fnExtractGatherList(ListFullPath, ref GatherFileList);
                }
                else
                {
                    PrintLog("ERR", "収集ファイルリストパスが指定されていません。収集機能は実行されません。");
                    return (int)CDefine.ERR_GATHER_NOTSET_TARGETFILE;//取得できない場合は終了

                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// 出力フォルダのパスチェック
        /// </summary>
        /// <returns></returns>
        public int CheckOutFolder()
        {
            try
            {
                string FolderPath = (string)Gather_FolerPath.Value;
                //出力フォルダの指定がある場合のみ実施
                if (FolderPath.Length > 0)
                {
                    string FullPath = FolderPath;
                    // フルパスの取得
                    if (CommonFunc.fnIsValidPath(ref FullPath) == false)
                    {
                        PrintLog("ERR", "出力フォルダパスが正しくありません。収集機能は実行されません。");
                        return (int)CDefine.ERR_GATHER_FATAL_OUTDIR;//取得できない場合は終了
                    }
                    // フォルダの存在チェック
                    if (Directory.Exists(FullPath) == false)
                    {
                        PrintLog("ERR", "出力フォルダパス["+ FullPath + "]が存在しません。収集機能は実行されません。");
                        return (int)CDefine.ERR_GATHER_FATAL_OUTDIR;//取得できない場合は終了
                    }
                }
                //else カレント
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// リストファイルからアーカイブ対象フォルダ・ファイルを抽出する
        /// </summary>
        /// <param name="srList">リストファイル</param>
        /// <param name="lsArchive">アーカイブ対象フォルダ・ファイル</param>
        private void fnExtractGatherList(string srFilePath, ref List<string> lsList)
        {
            using (StreamReader sr = new StreamReader(srFilePath, Encoding.UTF8))
            {
                string srLine = "";

                // EOFまで1行単位で読み込む
                while ((srLine = sr.ReadLine()) != null)
                {
                    // 空文字の場合は次の行へ
                    if ("" == srLine) { continue; }

                    // 不要な文字を取り除く
                    srLine = srLine.Trim();
                    srLine = srLine.TrimEnd(Path.DirectorySeparatorChar);

                    // 先頭が"#"の場合はコメント行と判断
                    if ("#" == srLine.Substring(0, 1)) { continue; }

                    // 読み込んだ情報をリストに追加
                    lsList.Add(srLine);
                }
            }

            return;
        }
        ///*****************************************************************************
        /// <summary>
        /// 出力スレッド情報生成
        /// </summary>
        /// <param name="usID"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public IO_TRDINFO CreateOutThreadInfo()
        {
            IO_TRDINFO trdInfo = new IO_TRDINFO();
            trdInfo.hTrigSem = new SemaphoreSlim(1, 1);         // 入力サービススレッド用トリガセマフォ
            trdInfo.threadCTS = new CancellationTokenSource();  // スレッド終了用token指定
            return trdInfo;
        }
        ///*****************************************************************************
        /// <summary>
        /// ログファイル収集スレッド
        /// </summary>
        /// <param name="cancelToken"></param>
        /// <param name="trdInfo"></param>
        ///*****************************************************************************
        public async void ThreadGatherService(CancellationToken cancelToken, IO_TRDINFO trdInfo)
        {
            try
            {
                if (GatherFileList.Count < 1)               //ファイルが登録されていない場合は処理しない
                {
                    PrintLog("ERR", "Stop OUT Service. Because Gather File is NULL.");
                    SetErrorID(CDefine.ERR_GATHER_NOTTARGETFILE);       // エラーインジケータをtrueに更新
                    return;
                }
                bool bSkipFirstFlag = (bool)SkipFirstFlag.Value; //1回目を飛ばすか否かのフラグを取得

                await Task.Run(async () =>
                {
                    while (true)
                    {
                        await trdInfo.hTrigSem.WaitAsync();
                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                            break;

                        //サイクル指定がある場合のみ処理を行う
                        if ((UInt64)Cycle.Value > 0)
                        {
                            if (bSkipFirstFlag == false)//飛ばさない、または2回目以降は処理を実行する
                            {
                                if (CheckInitEnd() == true && (bool)ServiceIndRun.Value == true) //Runがtrueの場合のみ実行
                                {
                                    fnGatherOn();//実処理
                                    ThreadCnt.Value = ++(trdInfo.uiOpeCnt);          //スレッド処理回数をTagへ書き込む
                                }
                            }
                            else//飛ばす場合はフラグをfalseにする
                                bSkipFirstFlag = false;

                            Thread.Sleep(TimeSpan.FromSeconds((UInt64)Cycle.Value));//秒でSleep
                            trdInfo.hTrigSem.Release();//終わったらリリースする事で実現
                        }
                    }
                });
            }
            catch (Exception ex)
            {
                PrintLog("ERR", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        
        /// <summary>
        /// Tagの値取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="Type"></param>
        /// <param name="defvalue"></param>
        /// <param name="target"></param>
        /// <returns></returns>
        public bool GetTagValue(string TagName, EGDEFINE.egTagDataType Type, object defvalue, ref object target)
        {
            try
            {
                target = defvalue;              //デフォルト値設定
                int ret = 0;                    //戻り値
                string strValue = "";           //string型取得時のテンポラリーオブジェクト
                object value = new object();    //string型以外取得時のテンポラリーオブジェクト

                //string型の場合は関数が違うため先に処理
                if (Type == EGDEFINE.egTagDataType.String)
                {
                    bool bRet = fnTagReadString(TagName, ref strValue);
                    if (bRet == false)
                        return false;
                    
                }
                else
                {
                    //それ以外は以下の関数
                    ret = EGAPI.EgTagRead(TagName, ref value);
                    if (ret != 0)
                    {
                        //メッセージは出さない 多くなるので。 PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));
                        return false;
                    }
                }
                switch (Type)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        target = (ret == 0) ? Convert.ToBoolean(value) : (Boolean)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.SByte:
                        target = (ret == 0) ? Convert.ToSByte(value) : (SByte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Byte:
                        target = (ret == 0) ? Convert.ToByte(value) : (Byte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int16:
                        target = (ret == 0) ? Convert.ToInt16(value) : (Int16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt16:
                        target = (ret == 0) ? Convert.ToUInt16(value) : (UInt16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int32:
                        target = (ret == 0) ? Convert.ToInt32(value) : (Int32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt32:
                        target = (ret == 0) ? Convert.ToUInt32(value) : (UInt32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int64:
                        target = (ret == 0) ? Convert.ToInt64(value) : (Int64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt64:
                        target = (ret == 0) ? Convert.ToUInt64(value) : (UInt64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Float:
                        target = (ret == 0) ? Convert.ToSingle(value) : (float)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Double:
                        target = (ret == 0) ? Convert.ToDouble(value) : (double)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.String:
                        target = (ret == 0) ? strValue : "";
                        break;
                    default:
                        break;
                }
                if (Type == EGDEFINE.egTagDataType.String)  //string型の場合とその他で変数が違うので分けて処理
                    PrintLog("DBG", String.Format("'{0}': {1}", TagName, strValue));
                else//それ以外は以下の関数
                    PrintLog("DBG", String.Format("'{0}': {1}", TagName, value));

                return true;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// タイプごとにタグの値を取得して設定 MntTagCls指定
        /// </summary>
        /// <param name="Type"></param>
        /// <param name="TagTailName"></param>
        /// <param name="defvalue"></param>
        public bool GetTagValueTagCls(ref MntTagCls myTag, object defvalue)
        {
            try
            {
                object target = new object();
                EGDEFINE.egTagDataType Type = myTag.Type;   //型取得
                string TagName = myTag.TagName;             //Tag名取得
                bool ret = GetTagValue(myTag.TagName, myTag.Type, defvalue, ref target);
                if (ret == true)
                    myTag.Value = target;
                return ret;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// Tagのアドレスを取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <returns></returns>
        public string GetTagAddress(string TagName)
        {
            string retValue = "";
            try
            {
                //アドレスの取得
                object retObj = new object();
                int ret = EGAPI.EgTagGetProperty(TagName, "Address", ref retObj);
                if (ret == 0)//取得できたら文字列に変換して返す
                {
                    retValue = retObj.ToString();
                    return retValue;
                }

                return retValue;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return retValue;
            }
        }
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool GetTagEntry(string TagName, ref IntPtr entry)
        {
            try
            {
                int ret = 0;        //戻り値

                object tagPtr = new object();
                ret = EGAPI.EgTagGetProperty(TagName, "Entry", ref tagPtr);
                if (ret == 0)
                {
                    entry = (IntPtr)tagPtr;
                    return true;
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool fnTagReadString(string TagName, ref string strValue)
        {
            try
            {
                int ret = 0;        //戻り値
                object strsize = 0;    //サイズ取得用
                ret = EGAPI.EgTagGetProperty(TagName, "Size", ref strsize);
                if (ret == 0)
                {
                    ret = EGAPI.EgTagReadString(TagName, ref strValue, (ushort)strsize);
                    if (ret == 0)
                        return true;
                    else
                        PrintLog("ERR", String.Format("'{0}': TagReadString Error[{1}].", TagName, ret));
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// 収集機能 開始
        /// </summary>
        public void fnGatherOn()
        {
            //
            //実行処理
            //
            if ((bool)ServiceIndRun.Value == true && GatherFileList.Count > 0)   //Runの時でかつファイルがある場合のみ実行
            {
                gather.Execute((string)Gather_FolerPath.Value, (string)Gather_ListPath.Value);
                ExecuteCnt.Value = (UInt32)ExecuteCnt.Value + (UInt32)1;    //統計情報カウント
            }
            return;
        }
        /// <summary>
        /// タグトリガー処理
        /// </summary>
        /// <param name="paramArray"></param>
        public void fnTagTriggerOn(byte[] paramArray)
        {
            try
            {
                if (EGAPI.EGFW.tagTriggerList.Count < 1) //タグトリガーの登録が無い場合は処理しない
                    return;

                // バイト配列の先頭2バイトをushort型に変換
                ushort tablecnt = BitConverter.ToUInt16(paramArray, 0);
                if (tablecnt == 0) //テーブル数 0の時は何もしない
                    return;
                if (tablecnt > 100) //100個以上は処理しない
                    return;

                // 構造体の配列に変換
                int structSize = Marshal.SizeOf<EGTAGTRIGINFO>();
                EGTAGTRIGINFO[] structArray = new EGTAGTRIGINFO[tablecnt];
                for (int i = 0; i < tablecnt; i++)
                {
                    byte[] structBytes = new byte[structSize];
                    Array.Copy(paramArray, (i * structSize) + 2, structBytes, 0, structSize); //3バイト目以降を変換
                    structArray[i] = ByteArrayToStructure<EGTAGTRIGINFO>(structBytes);
                    if (EGAPI.EGFW.tagTriggerList.Count > structArray[i].arrNo) //存在する場合のみ実施
                    {
                        if (structArray[i].var == 1) //値が1(True)の時のみ実行
                        {
                            //診断機能実行
                            fnGatherOn();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        // バイト配列から構造体に変換するヘルパーメソッド
        public static T ByteArrayToStructure<T>(byte[] bytes) where T : struct
        {
            GCHandle handle = GCHandle.Alloc(bytes, GCHandleType.Pinned);
            try
            {
                return Marshal.PtrToStructure<T>(handle.AddrOfPinnedObject());
            }
            finally
            {
                handle.Free();
            }
        }
        /// <summary>
        /// サービスメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                // 届いたEdgeメッセージに合わせて処理分岐しますよ
                switch (messNo)
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:            // サービス開始指令
                        // 初期化完了済みの場合のみ、処理実行します
                        if (CheckInitEnd())
                            StartService();            // サービス開始実行
                        else
                            SetInitStatus(CDefine.INIT_STS_INITRUN);    // 初期化中(RUN受信済み)に更新 ※初期化完了後にサービス開始されます
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:           // サービス終了指令
                        //Exe終了処理
                        Cleanup();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:          // サービス一時停止指令
                        //サービス停止指令
                        PauseService();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE:			// サービスアップデート指令 本コンテナ無効
                        break;
                    case (int)CDefine.EM_MAINTE_GATHER:                 // 収集機能トリガーメッセージ
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnGatherOn();//Gather On　処理
                        break;

                    case (int)EGDEFINE.EGMSG.EM_TAGTRIGGER_NOTIFICATION://タグトリガーメッセージ
                        //何かしら来た場合トリガー有りと判断
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnTagTriggerOn(paramArray);//TagTrigger 処理
                        break;
                    default:
                        return -1;
                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int PauseService()
        {
            try
            {
                ServiceIndRun.Value = false;                  // サービス停止状態に更新

                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// edge ファイナライズ処理
        /// </summary>
        public void EdgeFinalize()
        {
            // Liveカウンタタイマのストップ
            ServiceIndLiveTimer.RefleshStop();

            //収集スレッド終了処理
            if (ThreadInfo_Gather != null && ThreadInfo_Gather.hTrigSem != null)
                ThreadInfo_Gather.hTrigSem.Dispose();
            // RT-edgeフレームワーク終了
            EGAPI.EgFinalize();
        }
        /// <summary>
        /// サービス終了時の処理
        /// </summary>
        /// <returns></returns>
        public int Cleanup()
        {
            EdgeFinalize(); //edgeファイナライズ処理

            // プロセス終了
            var myself = Process.GetCurrentProcess();
            myself.CloseMainWindow();
            return 0;
        }
        /// <summary>
        /// ログ出力
        /// </summary>
        /// <param name="stServiceName"></param>
        /// <param name="stMode"></param>
        /// <param name="stFormatMess"></param>
        /// <param name="args"></param>
        public void PrintLog(string stMode, string stFormatMess, params string[] args)
        {
            string newFormatMess = stFormatMess.Replace("%", "%%"); //%が予期しないエラーになるためエスケープする
            try
            {
                edge_FW._PrintSysLog(GetServiceName(), stMode, newFormatMess, args);

            }
            catch//ログ出力に失敗時は無視
            {
            }
        }
    }
    /// <summary>
    /// .Live インジケータ更新用のタイマ
    /// </summary>
    public class ServiceLiveIndicatorTimer
    {
        bool IsStartF = false;
        EdgeService ServiceBody;
        System.Timers.Timer timer;

        public ServiceLiveIndicatorTimer(EdgeService _body, double n)
        {
            ServiceBody = _body;
            ServiceBody.ServiceIndLive.Value = (UInt32)0;
            timer = new System.Timers.Timer(n);
            timer.Elapsed += OnTimedEvent;
        }

        /// <summary>
        /// Liveインジケータの更新（タイマイベント）
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        private void OnTimedEvent(Object source, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                // インジケータ更新
                timer.Stop();
                ServiceBody.ServiceIndLive.Value = (UInt32)ServiceBody.ServiceIndLive.Value + (UInt32)1;

                // コンテナのステータスインジケータを更新、何かしらエラーがあれば .Error=true にします
                if ((UInt32)ServiceBody.ErrorID.Value != (UInt32)0)
                    ServiceBody.ServiceIndError.Value = (bool)true;
                else
                    ServiceBody.ServiceIndError.Value = (bool)false;
            }
            finally
            {
                // 次のタイマーイベントを許可
                timer.Start();
            }
        }
        object RefleshLock = new object();
        /// <summary>
        /// Liveインジケータの更新を開始
        /// </summary>
        public void RefleshStart()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                if (IsStartF == false)
                {
                    timer.Start();
                    IsStartF = true;
                }
            }
        }

        /// <summary>
        /// Liveインジケータの更新を停止
        /// </summary>
        public void RefleshStop()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                timer.Stop();
                IsStartF = false;
            }
        }
    }
}
