﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using RTedge;

namespace EgMReset
{
    /// <summary>
    /// RT-Edgeの操作クラス
    /// </summary>
    public partial class EdgeService
    {
        private static EdgeService service = new EdgeService(); // EdgeServiceクラス生成
        MainWindow _Main = null;                                // 親クラス
        Reset reset = null;                                     // 復旧クラス
        /// <summary>
        /// RT-edge container common
        /// </summary>
        public edge_API EGAPI;                                              // edge APIのインスタンス
        public string ServiceRealName = "";                                 // xmlで指定したサービスコンテナ名 EgInit後設定
        public string DefServiceName = "EgMReset";                          // サービスコンテナ名デフォルト値 EgInit前で使用
        public ServiceLiveIndicatorTimer ServiceIndLiveTimer;               // サービスインジケータ用のタイマ
        public byte InitState = CDefine.INIT_STS_IDLE;                      // サービス初期化状態(0:未初期化, 1:初期化中, 2:初期化中(Run受信済み), 3:初期化完了)

        
        
        /// <summary>
        /// コンストラクタ(private)
        /// </summary>
        private EdgeService() { }

        /// <summary>
        /// インスタンス取得関数
        /// --------------------------------------------------------
        /// 使用する際は下記のようにインスタンスを取得してください。
        /// EdgeService service = EdgeService.GetInstance();
        /// </summary>
        /// <returns></returns>
        public static EdgeService GetInstance()
        {
            return service;
        }
        /// <summary>
        /// サービス起動時処理
        /// </summary>
        /// <returns></returns>
        public int Startup(MainWindow Main)
        {
            try
            {
                SetInitStatus(CDefine.INIT_STS_INIT);           // 初期化中に更新

                _Main = Main;

                // RT-edgeの準備
                EGAPI = new edge_API();

                // RT-edgeユーザハンドラ登録
                edge_API.EDGECONFIG config = EGAPI.EDGE_CONFIG_DEFAULT;
                config.mInstMessFunc = MsgHandler;

                // RT-edgeフレームワーク初期化
                var ret = EGAPI.EgInit(config);
                if (ret == 0)
                {
                    // サービス名の設定
                    if (EGAPI.EGFW.ServiceRealName != null)
                        ServiceRealName = EGAPI.EGFW.ServiceRealName;

                    reset = new Reset();          // resetクラス生成

                    CreateMntTag();               //内部タグ構造体の作成

                    // ライブインジケータタグの周期機能 / Service Live counter timer
                    ServiceIndLiveTimer = new ServiceLiveIndicatorTimer(this, 1000);

                    SetErrorID(CDefine.SUCCESS);  //エラーコード設定

                    // インジケータタグの準備
                    CreateIndicater();

                    // XMLプロパティタグ設定の取り出し
                    GetProperty();

                    // Reset_ScriptPath ファイル存在確認 指定されている場合のみ存在チェック
                    if (((string)Reset_ScriptPath.Value).Length > 0 && File.Exists((string)Reset_ScriptPath.Value) == false)
                    {
                        PrintLog("ERR", "Failed to initialize. Script File does not exist. [" + (string)Reset_ScriptPath.Value + "]");
                        SetErrorID(CDefine.ERR_NOT_EXIST_PATH);  //エラーコード設定
                    }
                    else
                    {
                        ServiceIndLiveTimer.RefleshStart();
                        if (CheckInitRun())                                 // 初期化中(RUN受信済み)の場合
                        {
                            // 初期化途中に EM_SERVICE_RUN 受信していた場合は、ここに入る
                            SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                            StartService();                                 // サービス開始処理実行
                        }
                        SetInitStatus(CDefine.INIT_STS_INITEND);        // 初期化完了に更新
                    }
                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                SetErrorID((uint)0xFFFFFFFF);
                return -1;
            }
        }


        /// <summary>
        /// 自己のサービス名を返す
        /// </summary>
        /// <returns></returns>
        public String GetServiceName()
        {
            if (ServiceRealName.Length > 0) //XMLで指定したサービス名 EgInit後に設定 設定がない場合はデフォルト値を返す
                return ServiceRealName;
            return DefServiceName; //"EgBM"
        }
        /// <summary>
        /// サービス+タグ取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetServiceTagName(string TagTailName)
        {
            return String.Format("SERVICE.{0}.{1}", GetServiceName(), TagTailName);
        }
        /// <summary>
        /// システムタグ名取得
        /// </summary>
        /// <param name="TagTailName"></param>
        /// <returns></returns>
        public String GetSystemTagName(string TagTailName)
        {
            return String.Format("EDGESYSTEM.{0}", TagTailName);
        }
        /// <summary>
        /// 内部で保持するタグクラスの作成
        /// </summary>
        public void CreateMntTag()
        {
            //タグの定義
            ServiceIndRun = new MntTagCls(GetServiceTagName("Run"), EGDEFINE.egTagDataType.Boolean);　     // インジケータタグ".Run" : bool / Service Run indicator
            ServiceIndStatus = new MntTagCls(GetServiceTagName("Status"), EGDEFINE.egTagDataType.Byte);　  // インジケータタグ".Status" : byte / Service Status indicator
            ServiceIndError = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);　   // インジケータタグ".Error" : bool / Service Error indicator
            ServiceIndLive = new MntTagCls(GetServiceTagName("Live"), EGDEFINE.egTagDataType.UInt32);　    // インジケータタグ".Live" : UInt32 / Service Live counter indicator

            /// <summary>
            /// プロパティタグ
            /// </summary>
            AutoRun = new MntTagCls(GetServiceTagName("AutoRun"), EGDEFINE.egTagDataType.Boolean);             // プロパティタグ".AutoRun" : bool 動作自動スタートの有効有無
            Error = new MntTagCls(GetServiceTagName("Error"), EGDEFINE.egTagDataType.Boolean);               // インジケータタグ".Error" : bool エラー状態の有効有無
            ErrorID = new MntTagCls(GetServiceTagName("ErrorID"), EGDEFINE.egTagDataType.UInt32);             // インジケータタグ".ErrorID" UInt32 実行失敗時のエラーコード
            Mode = new MntTagCls(GetServiceTagName("Mode"), EGDEFINE.egTagDataType.Byte);                   // プロパティタグ".Mode" Byte モード
            Reset_ScriptPath = new MntTagCls(GetServiceTagName("ScriptPath"), EGDEFINE.egTagDataType.String);// プロパティタグ スクリプトパス String
        }
        /// <summary>
        /// 初期化状態更新
        /// </summary>
        /// <param name="sts"></param>
        public void SetInitStatus(byte sts)
        {
            InitState = sts;
        }
        /// <summary>
        /// 初期化完了済みか？(true:済み,false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitEnd()
        {
            return (InitState == CDefine.INIT_STS_INITEND);
        }
        /// <summary>
        /// 初期化中(RUN受信済み)か？(true:初期化中(RUN受信済み),false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitRun()
        {
            return (InitState == CDefine.INIT_STS_INITRUN);
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public int StartService()
        {
            try
            {
                ServiceIndRun.Value = true;                           // サービス実行状態に更新
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// ErrorIDの設定
        /// </summary>
        /// <param name="_ErrorID"></param>
        public void SetErrorID(uint _ErrorID)
        {
            ErrorID.Value = _ErrorID;
            if (_ErrorID == 0)  //0の時は成功 Errorフラグも変更
                ServiceIndError.Value = false;
            else
                ServiceIndError.Value = true;
        }
        /// <summary>
        /// インジケータタグの生成
        /// </summary>
        /// <returns></returns>
        public int CreateIndicater()
        {
            int ret = 0;

            // インジケータタグを生成します
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Status", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Error", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Byte, 1, "", "service status");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.ErrorID", GetServiceName()), (ushort)EGDEFINE.egTagDataType.UInt32, 4, "", "service etc last Error ID");
            ret |= EGAPI.EgTagCreate(String.Format("SERVICE.{0}.Run", GetServiceName()), (ushort)EGDEFINE.egTagDataType.Boolean, 1, "", "service runnning");

            //各値初期化
            ServiceIndError.Value = false;
            ErrorID.Value = (UInt32)0;
            
            return ret;
        }
        /// <summary>
        /// サービスプロパティのXML解釈処理
        /// </summary>
        /// <returns></returns>
        public bool GetProperty()
        {
            bool ret = true;
            ret |= GetTagValueTagCls(ref Mode, (byte)0);// Modeオプションの利用 0:何もしない 1:シャットダウン 2:再起動（デフォルト0 ）
            ret |= GetTagValueTagCls(ref Reset_ScriptPath, (string)"");// ScriptPathオプションの利用（デフォルト バッチプログラムデフォルト カレント プログラム名+lst）
            ret |= GetTagValueTagCls(ref AutoRun, (bool)false);//動作自動スタートの有効有無
            
            return ret;
        }
        
        /// <summary>
        /// Tagの値取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="Type"></param>
        /// <param name="defvalue"></param>
        /// <param name="target"></param>
        /// <returns></returns>
        public bool GetTagValue(string TagName, EGDEFINE.egTagDataType Type, object defvalue, ref object target)
        {
            try
            {
                target = defvalue;              //デフォルト値設定
                int ret = 0;                    //戻り値
                string strValue = "";           //string型取得時のテンポラリーオブジェクト
                object value = new object();    //string型以外取得時のテンポラリーオブジェクト

                //string型の場合は関数が違うため先に処理
                if (Type == EGDEFINE.egTagDataType.String)
                {
                    bool bRet = fnTagReadString(TagName, ref strValue);
                    if (bRet == false)
                        return false;
                    
                }
                else
                {
                    //それ以外は以下の関数
                    ret = EGAPI.EgTagRead(TagName, ref value);
                    if (ret != 0)
                    {
                        //メッセージは出さない 多くなるので。 PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));
                        return false;
                    }
                }
                switch (Type)
                {
                    case EGDEFINE.egTagDataType.Boolean:
                        target = (ret == 0) ? Convert.ToBoolean(value) : (Boolean)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.SByte:
                        target = (ret == 0) ? Convert.ToSByte(value) : (SByte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Byte:
                        target = (ret == 0) ? Convert.ToByte(value) : (Byte)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int16:
                        target = (ret == 0) ? Convert.ToInt16(value) : (Int16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt16:
                        target = (ret == 0) ? Convert.ToUInt16(value) : (UInt16)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int32:
                        target = (ret == 0) ? Convert.ToInt32(value) : (Int32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt32:
                        target = (ret == 0) ? Convert.ToUInt32(value) : (UInt32)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Int64:
                        target = (ret == 0) ? Convert.ToInt64(value) : (Int64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.UInt64:
                        target = (ret == 0) ? Convert.ToUInt64(value) : (UInt64)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Float:
                        target = (ret == 0) ? Convert.ToSingle(value) : (float)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.Double:
                        target = (ret == 0) ? Convert.ToDouble(value) : (double)defvalue;
                        break;
                    case EGDEFINE.egTagDataType.String:
                        target = (ret == 0) ? strValue : "";
                        break;
                    default:
                        break;
                }
                if (Type == EGDEFINE.egTagDataType.String)  //string型の場合とその他で変数が違うので分けて処理
                    PrintLog("DBG", String.Format("'{0}': {1}", TagName, strValue));
                else//それ以外は以下の関数
                    PrintLog("DBG", String.Format("'{0}': {1}", TagName, value));

                return true;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// タイプごとにタグの値を取得して設定 MntTagCls指定
        /// </summary>
        /// <param name="Type"></param>
        /// <param name="TagTailName"></param>
        /// <param name="defvalue"></param>
        public bool GetTagValueTagCls(ref MntTagCls myTag, object defvalue)
        {
            try
            {
                object target = new object();
                EGDEFINE.egTagDataType Type = myTag.Type;   //型取得
                string TagName = myTag.TagName;             //Tag名取得
                bool ret = GetTagValue(myTag.TagName, myTag.Type, defvalue, ref target);
                if (ret == true)
                    myTag.Value = target;
                return ret;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// Tagのアドレスを取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <returns></returns>
        public string GetTagAddress(string TagName)
        {
            string retValue = "";
            try
            {
                //アドレスの取得
                object retObj = new object();
                int ret = EGAPI.EgTagGetProperty(TagName, "Address", ref retObj);
                if (ret == 0)//取得できたら文字列に変換して返す
                {
                    retValue = retObj.ToString();
                    return retValue;
                }

                return retValue;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return retValue;
            }
        }
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool GetTagEntry(string TagName, ref IntPtr entry)
        {
            try
            {
                int ret = 0;        //戻り値

                object tagPtr = new object();
                ret = EGAPI.EgTagGetProperty(TagName, "Entry", ref tagPtr);
                if (ret == 0)
                {
                    entry = (IntPtr)tagPtr;
                    return true;
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        
        /// <summary>
        /// 文字列取得用関数 サイズが分からない為取得後読込
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="strValue"></param>
        /// <returns></returns>
        private bool fnTagReadString(string TagName, ref string strValue)
        {
            try
            {
                int ret = 0;        //戻り値
                object strsize = 0;    //サイズ取得用
                ret = EGAPI.EgTagGetProperty(TagName, "Size", ref strsize);
                if (ret == 0)
                {
                    ret = EGAPI.EgTagReadString(TagName, ref strValue, (ushort)strsize);
                    if (ret == 0)
                        return true;
                    else
                        PrintLog("ERR", String.Format("'{0}': TagReadString Error[{1}].", TagName, ret));
                }
                else
                    PrintLog("DBG", String.Format("'{0}': Not Defined XML.", TagName));

                return false;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// 復旧機能 開始
        /// </summary>
        public void fnResetOn()
        {
            //
            //実行処理
            //
            if((bool)ServiceIndRun.Value == true)   //Runの時のみ実行
                reset.Execute((byte)Mode.Value, (string)Reset_ScriptPath.Value);
            return;
        }
        /// <summary>
        /// タグトリガー処理
        /// </summary>
        /// <param name="paramArray"></param>
        public void fnTagTriggerOn(byte[] paramArray)
        {
            try
            {
                if (EGAPI.EGFW.tagTriggerList.Count < 1) //タグトリガーの登録が無い場合は処理しない
                    return;

                // バイト配列の先頭2バイトをushort型に変換
                ushort tablecnt = BitConverter.ToUInt16(paramArray, 0);
                if (tablecnt == 0) //テーブル数 0の時は何もしない
                    return;
                if (tablecnt > 100) //100個以上は処理しない
                    return;

                // 構造体の配列に変換
                int structSize = Marshal.SizeOf<EGTAGTRIGINFO>();
                EGTAGTRIGINFO[] structArray = new EGTAGTRIGINFO[tablecnt];
                for (int i = 0; i < tablecnt; i++)
                {
                    byte[] structBytes = new byte[structSize];
                    Array.Copy(paramArray, (i * structSize) + 2, structBytes, 0, structSize); //3バイト目以降を変換
                    structArray[i] = ByteArrayToStructure<EGTAGTRIGINFO>(structBytes);
                    if (EGAPI.EGFW.tagTriggerList.Count > structArray[i].arrNo) //存在する場合のみ実施
                    {
                        if (structArray[i].var == 1) //値が1(True)の時のみ実行
                        {
                            //復旧機能実行
                            fnResetOn();
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        // バイト配列から構造体に変換するヘルパーメソッド
        public static T ByteArrayToStructure<T>(byte[] bytes) where T : struct
        {
            GCHandle handle = GCHandle.Alloc(bytes, GCHandleType.Pinned);
            try
            {
                return Marshal.PtrToStructure<T>(handle.AddrOfPinnedObject());
            }
            finally
            {
                handle.Free();
            }
        }
        /// <summary>
        /// サービスメッセージハンドラ
        /// </summary>
        /// <param name="senderName"></param>
        /// <param name="messNo"></param>
        /// <param name="paramArray"></param>
        /// <returns></returns>
        public int MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                // 届いたEdgeメッセージに合わせて処理分岐しますよ
                switch (messNo)
                {
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:            // サービス開始指令
                        // 初期化完了済みの場合のみ、処理実行します
                        if (CheckInitEnd())
                            StartService();            // サービス開始実行
                        else
                            SetInitStatus(CDefine.INIT_STS_INITRUN);    // 初期化中(RUN受信済み)に更新 ※初期化完了後にサービス開始されます
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:           // サービス終了指令
                        //Exe終了処理
                        Cleanup();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:          // サービス一時停止指令
                        //サービス停止指令
                        PauseService();
                        break;
                    case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE:			// サービスアップデート指令 本コンテナ無効
                        break;
                    case (int)CDefine.EM_MAINTE_RESET:                 // 復旧機能トリガーメッセージ
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnResetOn();//Reset On　処理
                        break;

                    case (int)EGDEFINE.EGMSG.EM_TAGTRIGGER_NOTIFICATION://タグトリガーメッセージ
                        //何かしら来た場合トリガー有りと判断
                        if (CheckInitEnd()) //初期化終了後でないとNG
                            fnTagTriggerOn(paramArray);//TagTrigger 処理
                        break;
                    default:
                        return -1;
                }
                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int PauseService()
        {
            try
            {
                ServiceIndRun.Value = false;                  // サービス停止状態に更新

                return 0;
            }
            catch (Exception ex)
            {
                PrintLog("ERR", "FATAL ERROR : {0}", System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// edge ファイナライズ処理
        /// </summary>
        public void EdgeFinalize()
        {
            // Liveカウンタタイマのストップ
            ServiceIndLiveTimer.RefleshStop();

            // RT-edgeフレームワーク終了
            EGAPI.EgFinalize();
        }
        /// <summary>
        /// サービス終了時の処理
        /// </summary>
        /// <returns></returns>
        public int Cleanup()
        {
            EdgeFinalize(); //edgeファイナライズ処理

            // プロセス終了
            var myself = Process.GetCurrentProcess();
            myself.CloseMainWindow();
            return 0;
        }
        /// <summary>
        /// ログ出力
        /// </summary>
        /// <param name="stServiceName"></param>
        /// <param name="stMode"></param>
        /// <param name="stFormatMess"></param>
        /// <param name="args"></param>
        public void PrintLog(string stMode, string stFormatMess, params string[] args)
        {
            string newFormatMess = stFormatMess.Replace("%", "%%"); //%が予期しないエラーになるためエスケープする
            try
            {
                edge_FW._PrintSysLog(GetServiceName(), stMode, newFormatMess, args);

            }
            catch//ログ出力に失敗時は無視
            {
            }
        }
    }
    /// <summary>
    /// .Live インジケータ更新用のタイマ
    /// </summary>
    public class ServiceLiveIndicatorTimer
    {
        bool IsStartF = false;
        EdgeService ServiceBody;
        System.Timers.Timer timer;

        public ServiceLiveIndicatorTimer(EdgeService _body, double n)
        {
            ServiceBody = _body;
            ServiceBody.ServiceIndLive.Value = (UInt32)0;
            timer = new System.Timers.Timer(n);
            timer.Elapsed += OnTimedEvent;
        }

        /// <summary>
        /// Liveインジケータの更新（タイマイベント）
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        private void OnTimedEvent(Object source, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                // インジケータ更新
                timer.Stop();
                ServiceBody.ServiceIndLive.Value = (UInt32)ServiceBody.ServiceIndLive.Value + (UInt32)1;

                // コンテナのステータスインジケータを更新、何かしらエラーがあれば .Error=true にします
                if ((UInt32)ServiceBody.ErrorID.Value != (UInt32)0)
                    ServiceBody.ServiceIndError.Value = (bool)true;
                else
                    ServiceBody.ServiceIndError.Value = (bool)false;
            }
            catch (Exception ex)
            {
                string a = ex.ToString();
            }
            finally
            {
                // 次のタイマーイベントを許可
                timer.Start();
            }
        }
        object RefleshLock = new object();
        /// <summary>
        /// Liveインジケータの更新を開始
        /// </summary>
        public void RefleshStart()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                if (IsStartF == false)
                {
                    timer.Start();
                    IsStartF = true;
                }
            }
        }

        /// <summary>
        /// Liveインジケータの更新を停止
        /// </summary>
        public void RefleshStop()
        {
            lock (RefleshLock)  //1回だけスタートさせる
            {
                timer.Stop();
                IsStartF = false;
            }
        }
    }
}
