﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.IO;

namespace MtnGather
{
    public class FileOpe
    {
        /// <summary>
        /// パスタイプ
        /// </summary>
        public enum PathType
        {
            /// <summary>
            /// 該当なし
            /// </summary>
            None = 0,
            /// <summary>
            /// ファイル
            /// </summary>
            File,
            /// <summary>
            /// ディレクトリ
            /// </summary>
            Directory,
        }

        /// <summary>
        /// ファイルを削除する
        /// </summary>
        /// <param name="srDir">ファイル</param>
        /// <remarks>
        /// 標準FileクラスのDelete()では、読み取り専用フォルダ・ファイルを削除する際に例外発生する。
        /// そのため対象フォルダ内の全フォルダ・ファイルの読み取り専用属性を取り除いた後に削除する。
        /// </remarks>
        static public void fnDeleteFile(string srFile)
        {
            // ファイル情報を取得する
            FileInfo fi = new FileInfo(srFile);

            // ファイル属性を削除する
            fnRemoveFileAttr(fi);

            // ファイルを削除する
            fi.Delete();
        }

        /// <summary>
        /// ファイルの属性を削除する
        /// </summary>
        /// <param name="clFileInfo">ファイル情報</param>
        static public void fnRemoveFileAttr(FileInfo clFileInfo)
        {
            // 指定したファイルが読み取り専用の場合は削除する
            if ((clFileInfo.Attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
            {
                clFileInfo.Attributes = FileAttributes.Normal;
            }
        }
        /// <summary>
        /// フォルダを削除する
        /// </summary>
        /// <param name="srDir">フォルダ</param>
        /// <remarks>
        /// 標準DirectoryクラスのDelete()では、読み取り専用フォルダ・ファイルを削除する際に例外発生する。
        /// そのため対象フォルダ内の全フォルダ・ファイルの読み取り専用属性を取り除いた後に削除する。
        /// </remarks>
        static public void fnDeleteDir(string srDir)
        {
            // ディレクトリ情報の生成
            DirectoryInfo di = new DirectoryInfo(srDir);

            // ディレクトリ内の全てのフォルダ、ファイル属性を削除
            fnRemoveDirAttr(di);

            // ディレクトリ内の全てのフォルダ、ファイルを削除
            di.Delete(true);

        }
        /// <summary>
        /// フォルダ、ファイルの属性を削除する
        /// </summary>
        /// <param name="clDirInfo">ディレクトリ情報</param>
        static public void fnRemoveDirAttr(DirectoryInfo clDirInfo)
        {
            // フォルダの属性の削除
            if ((clDirInfo.Attributes & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
            {
                clDirInfo.Attributes = FileAttributes.Normal;
            }

            // ファイルの属性の削除
            foreach (FileInfo fi in clDirInfo.GetFiles())
            {
                fnRemoveFileAttr(fi);
            }

            // サブフォルダの属性を回帰的に変更
            foreach (DirectoryInfo di in clDirInfo.GetDirectories())
            {
                fnRemoveDirAttr(di);
            }
        }
        /// <summary>
        /// フォルダをコピーする
        /// </summary>
        /// <param name="srSrc">コピー元パス</param>
        /// <param name="srDst">コピー先パス</param>
        /// <param name="srPattern">検索文字列</param>
        static public void fnCopyDir(string srSrc, string srDst, string srPattern)
        {
            bool bIsExistDir = false;
            // フォルダ情報を取得する
            var dir = new DirectoryInfo(srSrc);



            // フォルダ内のファイルをコピーする
            foreach (FileInfo fi in dir.GetFiles(srPattern))
            {
                // コピー先のパスの生成
                var srDstFile = Path.Combine(srDst, fi.Name);

                if (bIsExistDir == false)
                {
                    // コピー先フォルダが存在しなければ作成
                    if (!Directory.Exists(srDst))
                    {
                        Directory.CreateDirectory(srDst);
                    }
                    bIsExistDir = true;
                }
                // ファイルコピー
                fi.CopyTo(srDstFile, true);
            }

            // サブフォルダを回帰的にコピー
            foreach (DirectoryInfo di in dir.GetDirectories())
            {
                var srDstDir = Path.Combine(srDst, di.Name);
                fnCopyDir(di.FullName, srDstDir, srPattern);
            }
        }

        /// <summary>
        /// ファイルをコピーする
        /// </summary>
        /// <param name="srSrc">コピー元パス</param>
        /// <param name="srDst">コピー先パス</param>
        static public void fnCopyFile(string srSrc, string srDst)
        {

            // コピー先フォルダの取得
            string srDir = Path.GetDirectoryName(srDst);

            // コピー先フォルダが存在しなければ作成
            if (!Directory.Exists(srDir))
            {
                Directory.CreateDirectory(srDir);
            }

            // ファイルのコピー
            File.Copy(srSrc, srDst, true);
        }


        /// <summary>
        /// ファイル名を変更する
        /// </summary>
        /// <param name="srFile">ファイルパス</param>
        static public void fnRenameFile(ref string srFile)
        {
            // 「_バックアップ日時(yyyymmddhhmmss).zip」で保存
            // 現在時刻の取得
            DateTime dtNow = DateTime.Now;
            // ファイルパスの分解：ディレクトリ
            string srDir = Path.GetDirectoryName(srFile);
            // ファイルパスの分解：ファイル名
            string srName = Path.GetFileNameWithoutExtension(srFile);
            // ファイルパスの分解：拡張子
            string srExt = Path.GetExtension(srFile);
            // ファイル名にバックアップ日時(yyyymmddhhmmss)を付加
            srName += "_" + dtNow.ToString("yyyyMMddHHmmss") + srExt;
            // 新しくファイルパスを生成する
            srFile = Path.Combine(srDir, srName);
        }


    }
}
