﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Text;
using System.IO;

namespace MtnGather
{
    public sealed class LogSave
    {
        private static LogSave logsave = new LogSave(); // ログセーブクラス生成

        /* ログレベル定数宣言 */
        public const UInt16 INF = 0x01;     // 任意のメッセージを出力します
        public const UInt16 CRT = 0x02;     // エラーメッセージを出力します
        public const UInt16 ERR = 0x04;     // CRTよりは重要度の低いエラーメッセージを出力します
        public const UInt16 DBG1 = 0x08;    // 開発者向けデバッグメッセージを出力します
        public const UInt16 DBG2 = 0x10;    // より詳細な開発者向けデバッグメッセージを出力します

        /*ログ関連定義*/
        //INFコード定義
        public static readonly uint EXE_START = 0x0000;                   //RTOS開始コード
        public static readonly uint EXE_STOP = 0x0001;                   //RTOS終了コード
        public static readonly uint RTOS_START = 0x1000;                   //RTOS開始コード
        public static readonly uint RTOS_STOP = 0x1001;                   //RTOS終了コード

        //ERR/DBG1コード定義
        public static readonly uint API_Err = 0x1000;                       //APIエラー
        public static readonly uint StartupArg = 0x2000;                   //起動引数コード
        public static readonly uint XMLFile = 0x4000;                       //xmlファイル読込みコード
        public static readonly uint TryCatch_Err = 0x5000;                  //TryCatchエラー
        public static readonly uint PARAMETER_Err = 0x6000;                 //パラメータエラー
        public static readonly uint INFO = 0x9000;                          //デバッグ情報

        //DGG2コード定義
        public static readonly uint IN_PARAMETER = 0x1000;           //FB入力パラメータメッセージコード
        public static readonly uint OUT_PARAMETER = 0x2000;           //FB出力パラメータメッセージコード
        public static readonly uint Function_StartEnd = 0x3000;           //関数開始・終了コード

        /* 変数定義 */
        private string szlogname;           // ログファイル名
        private string szlogdestpath;       // ログ出力先フォルダ
        private UInt16 uilevelfilter;       // ログレベルフィルター
        private UInt16 uirecord;            // ファイル出力行数上限

        private UInt16 uiLineCount = 0;     // 書込み行数カウンタ
        private UInt16 uiSameFileCount = 0; // 同名ファイル対策カウンタ
        private bool boInitComp = false;    // 初期化完了フラグ
        private bool boWriteNow = false;    // 書込み途中チェックフラグ
        private string szLogFileName;       // ログファイル名用([指定ファイル名]_yymmddhhmmss.log)
        private string szLogFilePath;       // ログファイルパス用(例:.\Log\[指定ファイル名]_yymmddhhmmss.log)

        object Look = new object();         //書込み処理排他制御用

        /// <summary>
        /// コンストラクタ(private)
        /// </summary>
        private LogSave() { }

        /// <summary>
        /// インスタンス取得関数
        /// --------------------------------------------------------
        /// 使用する際は下記のようにインスタンスを取得してください。
        /// LogSave logsave = LogSave.GetInstance();
        /// </summary>
        /// <returns></returns>
        public static LogSave GetInstance()
        {
            return logsave;
        }

        /// <summary>
        /// 初期化処理
        /// --------------------------------------------------------
        /// ログ出力クラスの初期化処理を行います。
        /// 設定値に誤りがある場合falseを返します。
        /// </summary>
        /// <param name="szLogName">ログファイル名称</param>
        /// <param name="szLogDestPath">ログ出力先のフォルダ</param>
        /// <param name="uiLevelFilter">ログの出力判断基準となるレベルフィルター</param>
        /// <param name="uiRecord">ファイルに出力するログ行数上限</param>
        /// <returns></returns>
        public bool Init(string szLogName, string szLogDestPath, UInt16 uiLevelFilter, UInt16 uiRecord)
        {
            try
            {
                //パラメータセット
                szlogname = szLogName;              // ログファイル名設定
                szlogdestpath = szLogDestPath;      // ログ出力先フォルダ設定
                uilevelfilter = uiLevelFilter;      // ログレベルフィルター設定
                uirecord = uiRecord;                // ログ行数上限設定

                //リセット処理
                uiLineCount = 0;    // 書込み行数カウンタリセット                
                boWriteNow = false; // 書込み途中チェックフラグリセット
                boInitComp = true;  // 初期化完了

                return true;        // 処理正常終了
            }
            catch
            {
                return false;       //例外発生
            }
        }

        /// <summary>
        /// ログ出力レベルを変更
        /// --------------------------------------------------------
        /// </summary>
        /// <param name="uiLevelFilter">ログの出力判断基準となるレベルフィルター</param>
        /// <returns></returns>
        public bool ResetLevelFilter(UInt16 uiLevelFilter)
        {
            try
            {
                //パラメータセット
                uilevelfilter = uiLevelFilter;      // ログレベルフィルター設定
                return true;        // 処理正常終了
            }
            catch
            {
                return false;       //例外発生
            }
        }

        /// <summary>
        /// ログ出力関数
        /// --------------------------------------------------------
        /// ログ出力処理を行います。
        /// 指定ログレベルが、書込み対象レベルならログファイルへ出力します。
        /// </summary>
        /// <param name="uiLevel">ログレベル</param>
        /// <param name="ulErrCode">エラーコード</param>
        /// <param name="szMsg">メッセージ</param>
        public int LogOutput(UInt16 uiLevel, UInt32 ulErrCode, string szMsg)
        {
            try
            {
                lock (Look)
                {
                    //初期化処理完了済みか？
                    if (!boInitComp)
                    {
                        return 1; // 未完了の場合ログ出力しない
                    }

                    //ログレベル判定(出力対象かチェック)
                    if ((uilevelfilter & uiLevel) == 0)
                    {
                        return 2; //レベルが出力対象ではないためログ出力しない
                    }

                    //ファイル行数が上限に到達しているか ?
                    if (uiLineCount >= uirecord) //上限に到達
                    {
                        boWriteNow = false; //書込み途中ファイルなし状態に遷移
                    }

                    //書込み途中のファイルがあるかチェック
                    if (boWriteNow == false)//書込み途中ファイルがない場合新規作成                               
                    {
                        uiLineCount = 0;    //書込み回数カウンタ初期化

                        DateTime dt = DateTime.Now;                             //現在時刻取得                    
                        DirectoryInfo di = new DirectoryInfo(szlogdestpath);    //フォルダ操作用インスタンス

                        // ログ出力先のフォルダが存在しない場合
                        if (di.Exists == false)
                        {
                            di.Create();    // フォルダを作成
                        }

                        szLogFileName = szlogname + "_" + dt.ToString("yyyyMMdd") + ".log";   //ログファイル名作成
                        szLogFilePath = Path.Combine(szlogdestpath, szLogFileName);                 //ログファイルパス作成


                        // 既に現在の日付のログファイルが存在するか？
                        if (File.Exists(szLogFilePath) == false)    //ファイルがない場合
                        {
                            uiSameFileCount = 0;                    // 同名ファイルカウンタ初期化
                            File.Create(szLogFilePath).Close();     // ログファイル新規作成
                        }
                        else //既にファイルがある場合
                        {
                            //ファイル行数取得
                            using (var reader = File.OpenText(szLogFilePath))
                            {
                                while (reader.ReadLine() != null)
                                {
                                    uiLineCount++;  //ファイル内行数カウントアップ
                                }

                                //ファイル行数が上限に到達しているか ?
                                if (uiLineCount >= uirecord) //上限に到達しているなら[test_yyyyMMddH(1).log]といった形式で別ファイルに書込む
                                {
                                    uiLineCount = 0;        // ファイル行数寛太リセット
                                    uiSameFileCount++;      // 同名ファイルカウンタ更新
                                    szLogFileName = szlogname + "_" + dt.ToString("yyyyMMdd") + "(" + uiSameFileCount + ").log";  //ログファイル名作成
                                    szLogFilePath = Path.Combine(szlogdestpath, szLogFileName);                                         //ログファイルパス作成
                                }
                            }
                        }

                        boWriteNow = true;  //ファイル書込み途中状態に遷移
                    }

                    //書き込み
                    WriteLog(szLogFilePath, uiLevel, ulErrCode, szMsg);
                }
                return 0;   //正常終了
            }
            catch
            {
                return 3;   //例外発生
            }
        }

        /// <summary>
        /// ログ書き込み
        /// --------------------------------------------------------
        /// 指定したファイルに以下の内容を追記します
        /// 現在の日付(yyyy/MM/dd HH:mm:ss),ログレベル情報,エラーコード,内容
        /// </summary>
        /// <param name="LogFilePath">ファイルパス</param>
        /// <param name="uiLevel">ログレベル</param>
        /// <param name="ulErrCode">エラーコード</param>
        /// <param name="cMsg">内容</param>
        private bool WriteLog(string LogFilePath, UInt16 uiLevel, UInt32 ulErrCode, string cMsg)
        {
            try
            {
                DateTime dt = DateTime.Now;     //現在時刻取得
                string szLevel;

                //レベル表示文字取得
                switch (uiLevel)
                {
                    case INF:
                        szLevel = "INF";
                        break;
                    case CRT:
                        szLevel = "CRT";
                        break;
                    case ERR:
                        szLevel = "ERR";
                        break;
                    case DBG1:
                        szLevel = "DBG1";
                        break;
                    case DBG2:
                        szLevel = "DBG2";
                        break;
                    default:
                        szLevel = "";
                        break;
                }

                // ファイル書き込み処理
                using (StreamWriter writer = new StreamWriter(LogFilePath, true, Encoding.GetEncoding("UTF-8")))
                {
                    string st = string.Format("{0},{1},0x{2:x4},{3}", dt, szLevel, ulErrCode, cMsg);  //書込み文字列作成 [時間,レベル,エラーコード,メッセージ]
                    writer.WriteLine(st);   //書込み
                    uiLineCount++;          //書込み行数カウントアップ
                }

                return true;
            }
            catch
            {
                return false;
            }

        }
    }
}
