﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
///---------------------------------------------------------------------------------
/// .iniファイルアクセス関数
///---------------------------------------------------------------------------------
using System;
using System.Runtime.InteropServices;
using System.Text;


namespace MtnReset
{
    public class IniFile
    {
        [DllImport("kernel32.dll")]
        private static extern int GetPrivateProfileString(
            string lpApplicationName,
            string lpKeyName,
            string lpDefault,
            StringBuilder lpReturnedstring,
            int nSize,
            string lpFileName);

        [DllImport("kernel32.dll")]
        private static extern int WritePrivateProfileString(
            string lpApplicationName,
            string lpKeyName,
            string lpstring,
            string lpFileName);

        string filePath;

        /// <summary>
        /// ファイル名を指定して初期化します。
        /// ファイルが存在しない場合は初回書き込み時に作成されます。
        /// </summary>
        public IniFile(string filePath)
        {
            this.filePath = filePath;
        }

        /// <summary>
        /// sectionとkeyからiniファイルの設定値を取得、設定します。 
        /// </summary>
        /// <returns>指定したsectionとkeyの組合せが無い場合は""が返ります。</returns>
        public string this[string section, string key]
        {
            set
            {
                WritePrivateProfileString(section, key, value, filePath);
            }
            get
            {
                StringBuilder sb = new StringBuilder(256);
                GetPrivateProfileString(section, key, string.Empty, sb, sb.Capacity, filePath);
                return sb.ToString();
            }
        }

        /// <summary>
        /// sectionとkeyからiniファイルの設定値を取得します。
        /// 指定したsectionとkeyの組合せが無い場合はdefaultvalueで指定した値が返ります。
        /// </summary>
        /// <returns>
        /// 指定したsectionとkeyの組合せが無い場合はdefaultvalueで指定した値が返ります。
        /// </returns>
        public string GetValue(string section, string key, string defaultvalue)
        {
            StringBuilder sb = new StringBuilder(256);
            GetPrivateProfileString(section, key, defaultvalue, sb, sb.Capacity, filePath);
            return sb.ToString();
        }
    }   // end of [class IniFile]

    //■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
    //■ clIniFileIo.cs(IniﾌｧｲﾙI/Oｸﾗｽ)本体
    //■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■■
    class clIniFileIo
    {
        // ■■■win32 API の IniﾌｧｲﾙI/O関数を利用する
        // ■初期化(Ini)ﾌｧｲﾙI/Oの為の初期化【Exe名からIniﾌｧｲﾙ名を指定して初期化】
        private IniFile inip = null;

        public clIniFileIo(string path)
        {
            inip = new IniFile(path);
        }

        //****************************************************************************
        //** ■■■Iniﾌｧｲﾙから取得した文字列の";"以降の削除と、その間のｽﾍﾟｰｽ削除(TAB削除は未対応)
        //**		vaVal	iniﾌｧｲﾙから取得した文字列
        //**		戻り値	指定文字列を処理した文字列
        //****************************************************************************
        private string fnStrSpt(string vaVal)
        {
            string vaRetrun;                                            // 文字列後尾の";"とｽﾍﾟｰｽ削除後の戻り値

            int iFind1 = vaVal.LastIndexOf(';');                        // 末尾から ';' を検索し、見つかった位置を取得する
            if (iFind1 > 0) vaRetrun = vaVal.Substring(0, iFind1);      // 1 文字目から n 文字の文字列を取得する（";"以降削除）
            else
                if (iFind1 == 0) vaRetrun = "";                         // 文字列の先頭に";"あるので文字列なし設定
            else
                vaRetrun = vaVal;                                   // ";"が無かったのでそのまま格納

            vaRetrun = vaRetrun.TrimEnd();                              // 文字列の後ろのｽﾍﾟｰｽを削除（TABは削除しません）

            return (vaRetrun);                                          // ▼編集結果を返す
        }   // end of fnStrSpt()

        //****************************************************************************
        //** ■■■Iniﾌｧｲﾙから文字列読込み（情報がない場合には指定の初期値を書込み）
        //**		(inip		iniﾌｧｲﾙへのﾘﾝｸﾊﾝﾄﾞﾙ)
        //**		vaSect		セクション名
        //**		vaKey		キー名
        //**		vaInitVal	初期値の文字列
        //**		戻り値		Iniﾌｧｲﾙから取得した文字列(後ろのｺﾒﾝﾄ部削除済み)
        //****************************************************************************
        public string fnIniStr(string vaSect, string vaKey, string vaInitVal)
        {
            try
            {
                string vaCsvPath = inip[vaSect, vaKey];                 // ｾｸｼｮﾝ、ｷｰ指定で文字列を取得
                if (vaCsvPath == "")                                    // 文字列は、設定されてないか？
                {
                    //inip[vaSect, vaKey] = vaInitVal;                    // Iniﾌｧｲﾙの初期値として指定の文字列を設定
                    vaCsvPath = vaInitVal;                              // 初期値を返す
                }
                vaCsvPath = fnStrSpt(vaCsvPath);                        // 文字列の後ろの";"以降のｺﾒﾝﾄを排除
                return (vaCsvPath);                                     // ▼値を返す
            }
            catch
            {
                return (vaInitVal);                                 // ▼ｴﾗｰ時は全て"初期値"を返す
            }
        }   // end of fnIniStr()

        //-----------------------------------------------------------------------------
        // 文字列をﾊﾞｲﾅﾘ値に変換して返す ("0x"が付く場合は16進で変換する)
        //	vaVal		：バイナリに変換する文字列
        //	vaInitVal	：変換エラー時の初期値
        //	戻り値		：バイナリに変換された値
        private int fnGetBin(string vaVal, int vaInitVal)
        {
            string va0x = "";                                           // 取得した文字列に"0x"があるかのﾏｰｸ
            try
            {
                if (vaVal.Length > 2)                                   // 3文字以上もの文字列か
                    va0x = vaVal.Substring(0, 2);                       // 先頭から２文字取出し

                if ((va0x == "0x") || (va0x == "0X"))
                    return (Convert.ToInt32(vaVal, 16));                // ▼文字列を16進で変換 );
                return (Convert.ToInt32(vaVal));                        // ▼文字列を10進で変換 );
            }
            catch
            {
                return (vaInitVal);                                     // ▼変換ｴﾗｰは初期値を返す
            }
        }   // end of fnGetBin()

        //-----------------------------------------------------------------------------
        // 初期値の値をint型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数（ｺﾒﾝﾄの付加も対応ﾀｲﾌﾟ）
        //	vaVal		：バイナリに変換する文字列
        //	vaInitVal	：キー無し時の初期値
        //	vaComment	：Iniファイルに値の後に付かされるコメント(";"不要)：キーが無い場合の追加時
        //	戻り値		：バイナリに変換された値
        public int fnIniStr(string vaSect, string vaKey, int vaInitVal, string vaComment)
        {
            try
            {
                int iFind1 = vaComment.LastIndexOf(';');    // 末尾から ';' を検索し、見つかった位置を取得する
                if (iFind1 < 0)                             // ";"無かったか？（先頭に有るときは、0 となる）
                    vaComment = " ;" + vaComment;           // ”;"を文字列前に追加すつ

                // 初期値を10進文字列に変換してIniﾌｧｲﾙの読込み要求
                string vaVal = fnIniStr(vaSect, vaKey, vaInitVal.ToString() + vaComment);
                if (vaVal.Length == 0) return (vaInitVal);              // ▼初期値を返す
                return (fnGetBin(vaVal, vaInitVal));                        // ▼文字列を10/16進で変換 );
            }
            catch
            {
                return (vaInitVal);                                     // ▼変換ｴﾗｰは初期値を返す
            }
        }

        //-----------------------------------------------------------------------------
        // 初期値の値をint型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数（ｺﾒﾝﾄ付加未対応ﾀｲﾌﾟ）
        //	vaVal		：バイナリに変換する文字列
        //	vaInitVal	：キー無し時の初期値
        //	戻り値		：バイナリに変換された値
        public int fnIniStr(string vaSect, string vaKey, int vaInitVal)
        {
            try
            {
                // 初期値を10進文字列に変換してIniﾌｧｲﾙの読込み要求
                string vaVal = fnIniStr(vaSect, vaKey, vaInitVal.ToString());
                if (vaVal.Length == 0) return (vaInitVal);              // 初期値を返す
                return (fnGetBin(vaVal, vaInitVal));                        // ▼文字列を10/16進で変換 );
            }
            catch
            {
                return (vaInitVal);                                     // ▼変換ｴﾗｰは初期値を返す
            }
        }

        //-----------------------------------------------------------------------------
        // 初期値の値をint型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数
        //	vaVal		：バイナリに変換する文字列
        //	vaInitVal	：キー無し時の初期値
        //	n16			：=16は、16進変換で初期値を設定
        //	戻り値		：バイナリに変換された値
        public int fnIniStr(string vaSect, string vaKey, int vaInitVal, int n16)
        {
            string vaTyp;

            if (n16 == 16)
                vaTyp = "0x" + vaInitVal.ToString("X4");                // 16進で初期値を作る
            else
                vaTyp = vaInitVal.ToString();                           // 10進で初期値を作る

            string vaVal = fnIniStr(vaSect, vaKey, vaTyp);              // Iniﾌｧｲﾙからの読込み
            if (vaVal.Length == 0) return (vaInitVal);                  // ▼初期値を返す

            return (fnGetBin(vaVal, vaInitVal));                            // ▼文字列を10/16進で変換 );
        }

        //-----------------------------------------------------------------------------
        // 初期値の値をint型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数
        // n16=16は、16進変換で初期値を設定＋ｺﾒﾝﾄの付加にも対応版
        //	vaVal		：バイナリに変換する文字列
        //	vaInitVal	：キー無し時の初期値
        //	n16			：=16は、16進変換で初期値を設定
        //	vaComment	：Iniファイルに値の後に付かされるコメント(";"不要)：キーが無い場合の追加時
        //	戻り値		：バイナリに変換された値
        public int fnIniStr(string vaSect, string vaKey, int vaInitVal, int n16, string vaComment)
        {
            string vaTyp;

            if (n16 == 16)
                vaTyp = "0x" + vaInitVal.ToString("X4");                // 16進で初期値を作る
            else
                vaTyp = vaInitVal.ToString();                           // 10進で初期値を作る

            int iFind1 = vaComment.LastIndexOf(';');                    // 末尾から ';' を検索し、見つかった位置を取得する
            if (iFind1 < 0)                                             // ";"無かったか？（先頭に有るときは、0 となる）
                vaComment = " ;" + vaComment;                           // ”;"を文字列前に追加すつ

            string vaVal = fnIniStr(vaSect, vaKey, vaTyp + vaComment);  // Iniﾌｧｲﾙからの読込み
            if (vaVal.Length == 0) return (vaInitVal);                  // ▼初期値を返す

            return (fnGetBin(vaVal, vaInitVal));                            // ▼文字列を10/16進で変換 );
        }

        //****************************************************************************
        //** ■■■Iniﾌｧｲﾙへ文字列書き込み(現在のｺﾒﾝﾄは残したままで更新します。)
        //**		(inip		iniﾌｧｲﾙへのﾘﾝｸﾊﾝﾄﾞﾙ)
        //**		vaSect		セクション名
        //**		vaKey		キー名
        //**		vaInitVal	初期値の文字列
        //****************************************************************************
        public int fnIniStw(string vaSect, string vaKey, string vaInitVal)
        {
            try
            {
                string vaKeyVal = inip[vaSect, vaKey];                  // ｾｸｼｮﾝ、ｷｰ指定で現在の文字列を取得
                if (vaKeyVal != "")                                     // 文字列は、設定されているか？
                {
                    int iFind1 = vaKeyVal.LastIndexOf(';');             // 末尾から ';' を検索し、見つかった位置を取得する
                    if (iFind1 >= 0)                                    // ";"発見か？（先頭に有るときは、0 となる）
                    {
                        string vaCsvVal = fnStrSpt(vaKeyVal);           // 現在のキー値
                        iFind1 = vaCsvVal.Length;
                        if (iFind1 == 0)
                            vaKeyVal = " " + vaKeyVal;                  // キーが無い場合の対応で ";" の前に空白を入れる
                        else
                            vaKeyVal = vaKeyVal.Substring(iFind1, vaKeyVal.Length - iFind1);    // 現在の空白(TABも)＋ｺﾒﾝﾄ部の取出し
                        vaKeyVal = fnStrSpt(vaInitVal) + vaKeyVal;      // 現在のｺﾒﾝﾄ部を追加して書込みデータとする
                    }
                    else
                        vaKeyVal = vaInitVal;                           // ﾊﾟﾗﾒｰﾀで取得した値をそのまま書込みデータとする
                }
                else
                    vaKeyVal = vaInitVal;                               // ﾊﾟﾗﾒｰﾀで取得した値をそのまま書込みデータとする

                inip[vaSect, vaKey] = vaKeyVal;                         // 指定の文字列をIniﾌｧｲﾙに書込み(これまでのｺﾒﾝﾄ部を残して更新)
                return (0);                                             // ▼正常にIniﾌｧｲﾙに設定した場合
            }
            catch
            {
                return (-1);                                            // ▼Iniﾌｧｲﾙへの設定ｴﾗｰは"-1"で返す
            }
        }   // end of fnIniStw()

        //-------------------------------------------------------------------------------------------------
        // 初期値の型を int型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数
        //**		vaSect		セクション名
        //**		vaKey		キー名
        //**		vaInitVal	初期値
        public int fnIniStw(string vaSect, string vaKey, int vaInitVal)
        {
            return (fnIniStw(vaSect, vaKey, vaInitVal.ToString()));     // ▼10進でIniﾌｧｲﾙに設定
        }

        //-------------------------------------------------------------------------------------------------
        // 初期値の型を int型にしたｵｰﾊﾞｰﾛｰﾄﾞ関数
        //**		vaSect		セクション名
        //**		vaKey		キー名
        //**		n16			=16は、16進変換(0x付き)でIniﾌｧｲﾙに値を設定する
        //**		vaInitVal	初期値
        public int fnIniStw(string vaSect, string vaKey, int vaInitVal, int n16)
        {
            string vaTyp;

            if (n16 == 16)
                vaTyp = "0x" + vaInitVal.ToString("X4");                // 16進で初期値を作る
            else
                vaTyp = vaInitVal.ToString();                           // 10進で初期値を作る

            return (fnIniStw(vaSect, vaKey, vaTyp));                    // ▼10/16進でIniﾌｧｲﾙに設定
        }

    }	// end of class clIniFileIo()
}
